#include "acmd.h"
#include <QTreeWidget>
#include <QTreeWidgetItem>
#include <QMultiMap>
#include <AlarmTreeWidget.h>
#include <AlarmTreeWidgetItem.h>
#include <quwriter.h>
#include <QtDebug>

Acmd::Acmd() {

}

// ack alarms
// if a selected element represents a server error (not a true alarm), then do not operate on it
// and store the name into errdevs
void Acmd::ack(const QTreeWidget *tw, bool all, CumbiaPool *cup, const CuControlsFactoryPool& fpoo, QStringList &errdevs) {
    QMultiMap<QString, QString> map = m_getDevAlarmMap(tw, all);
    foreach(const QString& dev, map.uniqueKeys()) {
        QuWriter *w = new QuWriter(nullptr, cup, fpoo);
        const QStringList &alarms = map.values(dev);
        if(alarms.size() != 1 || alarms[0].count('/')  == 0) { // exclude "Alarm server error"
            w->setAutoDestroy(true);
            w->setTarget(QString("%1->Ack").arg(dev));
            w->execute(alarms);

            /* stop new */
            QuWriter *stopNewWriter = new QuWriter(nullptr, cup, fpoo);
            stopNewWriter->setAutoDestroy(true);
            stopNewWriter->setTarget(dev + "->StopNew");
            stopNewWriter->execute();
        }
        else
            errdevs << alarms[0];
    }
}

// silence alarms
// if a selected element represents a server error (not a true alarm), then do not operate on it
// and store the name into errdevs
void Acmd::silence(const QTreeWidget *tw, bool all, CumbiaPool *cup, const CuControlsFactoryPool &fpoo, QStringList &errdevs) {
    QMultiMap<QString, QString> map = m_getDevAlarmMap(tw, all);
    foreach(const QString& dev, map.uniqueKeys()) {
        QuWriter *w = new QuWriter(nullptr, cup, fpoo);
        const QStringList &alarms = map.values(dev);
        if(alarms.size() != 1 || alarms[0].count('/')  == 0) { // exclude "Alarm server error"
            w->setAutoDestroy(true);
            w->setTarget(dev + "->Silence");
            w->execute(alarms);
        }
        else
            errdevs << alarms[0];
    }
}

// enable selected alarms
//
void Acmd::enable(const QTreeWidget *tw, CumbiaPool *cup, const CuControlsFactoryPool &fpoo) {
    QMultiMap<QString, QString> map = m_getDevAlarmMap(tw, false);
    foreach(const QString& dev, map.uniqueKeys()) {
        QuWriter *w = new QuWriter(nullptr, cup, fpoo);
        const QStringList &alarms = map.values(dev);
        if(alarms.size() != 1 || alarms[0].count('/') == 0) { // exclude "Alarm server error"
            w->setAutoDestroy(true);
            w->setTarget(dev + "->Enable");
            w->execute(alarms);
        }
    }
}

// stop new alarms
// if a selected element represents a server error (not a true alarm), then do not operate on it
// and store the name into errdevs
void Acmd::stop_new(const QTreeWidget *tw, bool all, CumbiaPool *cup, const CuControlsFactoryPool &fpoo,
                    QStringList &errdevs) {
    QMultiMap<QString, QString> map = m_getDevAlarmMap(tw, all);
    foreach(const QString& dev, map.uniqueKeys()) { // exclude "Alarm server error"
        QuWriter *w = new QuWriter(nullptr, cup, fpoo);
        const QStringList &alarms = map.values(dev);
        if(alarms.size() != 1 || alarms[0].count('/') == 0) {
            w->setAutoDestroy(true);
            w->setTarget(dev + "->StopNew");
            w->execute(alarms);
        }
        else
            errdevs << alarms[0];
    }
}

QMultiMap<QString, QString> Acmd::m_getDevAlarmMap(const QTreeWidget *tw, bool all) const {
    QMultiMap<QString, QString> map;
    QList<QTreeWidgetItem *> selected;
    for(int i = 0; all && i < tw->topLevelItemCount(); i++)
        selected << tw->topLevelItem(i);
    if(!all)
        selected = tw->selectedItems();
    foreach(QTreeWidgetItem *si, selected)
        map.insert(si->data(AlarmTreeWidgetItem::Device, Qt::UserRole).toString(), si->text(AlarmTreeWidget::ALARM));
    return map;
}
