/***************************************************************************
 *   Copyright (C) 2007 by Claudio Scafuri, Giacomo Strangolino   *
 *   claudio@hyo   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef alarm_H
#define alarm_H

#include "ui/ui_alarm.h"
#include <vector>
#include <QString>
#include <QHash>
#include <QMap>
#include <tango.h>
#include "Sound.h"

// cumbia
#include <qulogimpl.h>
#include <cucontrolsfactorypool.h>
class CumbiaPool;
class CuData;
// cumbia

class AlarmFilter;
class AlarmSQLHandle;
class AlarmHistoryTree;
class AlarmTreeWidgetItem;
class QTreeWidgetItem;

class Alarm: public QWidget
{
   Q_OBJECT

public:
   Alarm(CumbiaPool *cu_p, QWidget *);
   ~Alarm();

public slots:

   void queryDB();
protected slots:
   void refresh(const CuData &da);
   void onDevList(const CuData& da);
   void onGroupList(const CuData &da);
   void m_connect_devs(const QStringList &devs);

   void ack();
   void ackAll();
   void silence();
   void enable();
   void stopSound();
   void testSound();
   void tabChanged(int);
   void onInfoListViewVisibilityChange(bool vis);

   void onItemSelectionChange();
   void onItemDoubleClicked(const QModelIndex &mi);
   void onFilterNameChanged(const QString& n);
   void onFilterStatusChanged(const QString& s);
   void onFilterStatusEnabled(bool en);

   void beep();

   void openUrl(const QString& u) const;

protected:
   void closeEvent(QCloseEvent *);

private slots:
   void enableFrameHistoryInterval(int i);
private:
   void insertItem(const QStringList &s, int index);
   void tangoStringListCommand(const QString &device_name, const QStringList &names, const char *cmdname);
   void mRemoveStaleItems(const QSet<QString> stale_names);
   QList<QTreeWidgetItem *> mGetSelectedItems();

   Ui::Alarm *ui;
   Sound *sound;
   bool filtering;
   AlarmFilter *m_filter;

   // cumbia
   CumbiaPool *cu_pool;
   QuLogImpl m_log_impl;
   CuControlsFactoryPool m_ctrl_factory_pool;
   QStringList m_handlers;
   // cumbia
};


#endif
